@@ -25,6 +25,8 @@
 
 #include <boost/program_options.hpp>
 
+#include "config.h"
+
 #include "setup.h"
 #include "cryptengine.h"
 #include "cryptengine_nettle.h"
@@ -82,7 +84,7 @@ read_options(int argc, char *argv[], std::string& cmd, apso::Options& options) {
 
 	// Informational commands (help, version & list-vc):
 	if (vm.count("help")) {
-		std::cout << "Apso v.0.0.2\n";
+		std::cout << PACKAGE_STRING << "\n";
 		std::cout << "This is a brief list of commands. For more detailed information,\n"
 			  << "please see the info file.\n\n";
 		std::cout << desc << "\n";
@@ -90,7 +92,7 @@ read_options(int argc, char *argv[], std::string& cmd, apso::Options& options) {
 	}
 
 	if (vm.count("version")) {
-		std::cout << "Apso v.0.0.2\n";
+		std::cout << PACKAGE_STRING << "\n";
 		exit (0);
 	}
 
