@@ -22,6 +22,8 @@
 
 #include <boost/filesystem/fstream.hpp>
 
+#include <gmpxx.h>
+
 extern "C" {
 #include <fcntl.h>
 #include <unistd.h>
@@ -34,18 +36,34 @@ extern "C" {
 #include <nettle/base16.h>
 #include <nettle/cbc.h>
 #include <nettle/nettle-types.h>
+#include <nettle/yarrow.h>
+#include <nettle/rsa.h>
+#include <nettle/bignum.h>
 }
 
+
 #include "cctools.h"
 #include "cryptengine_nettle.h"
 
-extern "C" {
-uint8_t * __cryptengine_nettle_asym_enc (const uint8_t *text, const uint8_t *pubk, unsigned *size, const unsigned keysize);
-uint8_t * __cryptengine_nettle_asym_dec (const uint8_t *text, const uint8_t *prik, unsigned *size, const unsigned keysize);
-}
+
+// FIXME: RANDOM_DEVICE should actually be "platform::RANDOM_DEVICE",
+// so we can have "platform" being Linux, BSD, Windows, etc.
+#define RANDOM_DEVICE "/dev/urandom"
 
 namespace apso {
 
+
+// Static members:
+struct yarrow256_ctx CryptEngineNettle::yarrow;
+
+
+
+/**
+ * Allocates a buffer and fills it with random bits.
+ *
+ * @param size The size of the buffer to be allocated
+ * @return The buffer, filled with random bits
+ */
 char * 
 CryptEngineNettle::random_bytes   (const unsigned size) 
 	throw (std::runtime_error, std::bad_alloc) {
@@ -59,13 +77,49 @@ CryptEngineNettle::random_bytes   (const unsigned size)
 	return buffer;
 }
 
-void
-CryptEngineNettle::random_fill    (char *buffer, const unsigned size) 
+
+/**
+ * Constructor for Nettle engine.
+ *
+ * This will initialize the RNG.
+ *
+ * FIXME: The RNG should reseed periodically, so the constructor 
+ * should use two sources of entropy and use them to update the pools:
+ * yarrow256_init   (yarrow&, 2, char ** sources);
+ * Later the random_fill method can do:
+ * yarrow256_update (yarrow&, source_number, 0, length, data);
+ */
+CryptEngineNettle::CryptEngineNettle()
 	throw (std::runtime_error) {
-	// FIXME: Use Nettle's Yarrow implementation
-	__random_fill((uint8_t *) buffer,size);
+        unsigned source_size = 50;
+        unsigned length;
+        char * seedbuffer = (char*) malloc (source_size*sizeof(char));
+        yarrow256_init (& CryptEngineNettle::yarrow, 0, NULL);
+
+	boost::filesystem::ifstream in (RANDOM_DEVICE);
+        in.read(seedbuffer, source_size);
+        length=in.gcount();
+        if (length == 0)
+                throw std::runtime_error ("Can't initialize RNG: RANDOM_DEVICE returned zero bytes");
+
+        yarrow256_seed (& CryptEngineNettle::yarrow, length, (uint8_t *)seedbuffer);
+        free(seedbuffer);
 }
 
+/**
+ * Fills a buffer with random bits.
+ *
+ * FIXME: This is dangerous. Maybe there should be a public method that
+ * fills in a bdata object, and a private one that operates on char* ?
+ *
+ * @param buffer The buffer to be filled.
+ * @param size The size of the buffer.
+ */
+void
+CryptEngineNettle::random_fill(char *buffer, const unsigned size)
+	throw (std::runtime_error) {
+	yarrow256_random (& CryptEngineNettle::yarrow, size, (uint8_t *)buffer);
+}
 
 /**
  * Symmetric encryption using AES.
@@ -208,11 +262,41 @@ CryptEngineNettle::sym_dec (const char *enctext, const char *key, unsigned *size
 char *
 CryptEngineNettle::asym_enc (const char *text, const char *pubk, unsigned *size, const unsigned keysize)
 	throw (std::bad_alloc,std::runtime_error) {
+
 	unsigned new_size = *size;
-	uint8_t * pre_result = (uint8_t *) __cryptengine_nettle_asym_enc ((uint8_t *)text,
-									  (uint8_t *)pubk,
-									  &new_size, 
-									  (unsigned) keysize);
+
+	// Copy the input:
+	uint8_t * copytext = (uint8_t *) malloc (*size * sizeof(uint8_t));
+	memcpy(copytext,text,*size);
+
+	uint8_t random_data[20];    
+	struct rsa_public_key key;
+	mpz_t x;
+
+
+	random_fill ((char *)random_data, 20);
+	
+	rsa_public_key_init(&key);
+	if (!rsa_keypair_from_sexp(&key, NULL, 0, keysize, (uint8_t *)pubk)) {
+		throw std::runtime_error ("Can't read public key.");
+	}
+	random_fill ((char *)copytext, *size);
+	mpz_init(x);
+
+	if (!rsa_encrypt(&key,
+			 & CryptEngineNettle::yarrow,
+			 (nettle_random_func) yarrow256_random,
+			 *size,
+			 (uint8_t *)text,
+			 x)) {
+			 throw std::runtime_error ("RSA_ENCRYPT FAILED");
+	}
+	rsa_public_key_clear(&key);
+
+	new_size = nettle_mpz_sizeinbase_256_u (x);
+	uint8_t * pre_result = (uint8_t *) malloc (new_size * sizeof(uint8_t));
+	nettle_mpz_get_str_256(new_size, pre_result, x);
+
 	if (pre_result == 0) {
 		throw std::runtime_error ("CryptEngineNettle::asym_enc: Could not RSA-encrypt.");
 	}
@@ -234,8 +318,24 @@ CryptEngineNettle::asym_dec (const char *enctext, const char *prik, unsigned *si
 
         unsigned decoded_size = *size;
         uint8_t * decoded_text = (uint8_t *) decode ((const char *)enctext, &decoded_size);
-	std::string d ((char *)decoded_text, decoded_size);
-	uint8_t * result = __cryptengine_nettle_asym_dec (decoded_text, (uint8_t *)prik, &decoded_size, keysize);
+
+	uint8_t * result = (uint8_t *) malloc ((decoded_size) * sizeof(uint8_t *));
+
+	mpz_t x;
+	struct rsa_private_key key;
+        rsa_private_key_init(&key);
+	if (!rsa_keypair_from_sexp(NULL, &key, 0, keysize, (uint8_t *)prik))
+		throw std::runtime_error ("Can't read private key");
+	
+	nettle_mpz_init_set_str_256_u(x, decoded_size, decoded_text);
+        nettle_mpz_set_str_256_u(x, decoded_size, decoded_text);
+
+	unsigned length = decoded_size;
+        int res;
+	if (! (res = rsa_decrypt(&key, &length, result, x)) )
+		// fprintf(stderr,"Can't decrypt. res = %d, length = %d, size = %d\n",res,length,*size);
+		throw std::runtime_error ("Can't decrypt.");
+	*size = length;
 
 	if (result == 0)
 		throw std::runtime_error("CryptEngineNettle::asym_dec: Could not RSA-decrypt.");
