@@ -34,18 +34,17 @@
 
 namespace apso {
 
-extern "C" {
-int __random_fill (uint8_t *buffer, unsigned n);
-}
 
 /**
  * Wrappers over cryptographic functions for the Nettle library.
  */
 class CryptEngineNettle : public CryptEngine {
 private:
+	static struct yarrow256_ctx yarrow;
 	int base64_dec (char ** result, const char *text, const long size);
 	int base64_enc (char ** result, const char *text, const long size);
 public:
+	CryptEngineNettle     () throw (std::runtime_error);
 	char * random_bytes   (const unsigned size)                     throw(std::runtime_error, std::bad_alloc);
 	void   random_fill    (char *buffer, const unsigned size)       throw(std::runtime_error);
 
